#!/usr/local/bin/perl
#1_find_orfs.pl
#Author: Paul Stothard
#
#This script submits a sequence to NCBI's ORF finder and writes the
#html output to a file. It then parses the html file and prints out
#information about the ORFs that were found.

use strict;
use warnings;

#Perl comes with a group of modules that can be used to automate
#Web-related activities. We need to load two modules that we will be
#using.
use LWP::UserAgent;
use HTTP::Request::Common;

#Obtain the DNA sequence from a file.
print "Please enter the name of a sequence file"
    . " (try sars.txt) in FASTA format and press enter.\n";

my $seq_filename = <STDIN>;
chomp($seq_filename);
local $/ = undef;
open( my $SEQFILE, $seq_filename )
    or die("Cannot open file '$seq_filename': $!");
my $sequence = <$SEQFILE>;
close($SEQFILE);

#Create a user agent object of class LWP::UserAgent. The user agent
#object acts like a web browser, and we can use it to send requests
#for resources on the web. The user agent object returns the results
#of these requests to us in the form of a response object. In the
#following statement we are creating the user agent object using the
#"LWP::UserAgent->new()". We refer to this object using $browser. Thus
#we have created a virtual web browser that we can access using a name
#we gave it--$browser.
my $browser = LWP::UserAgent->new();

#Our virtual web browser has many attributes we can adjust. A list of
#these attributes is available at
#http://search.cpan.org/author/GAAS/libwww-perl/lib/LWP/UserAgent.pm
#We will change the timeout value of our browser, which is how long
#the browser will wait for a response before timing out (i.e.,
#canceling the request). We will set it to 30 seconds.
$browser->timeout(30);

#Send a requrest to the NCBI ORF Finder server. A request contains the
#information that a normal web browser sends to a server when it makes
#a request for a particular page. This information includes the URL of
#the page. In our case we also need to submit the information that the
#NCBI ORF Finder server expects. The following statement causes our
#virtual browser to send a request. The results of the request, the
#response, is stored in $response. The part of the request that the
#ORF server sees is the list of attributes and values, which are
#SEQUENCE => $sequence, and gcode => '1'. The value 1 is used to
#indicate the standard genetic code. These attributes were identified
#by reading the page source of the NCBI ORF Finder web site at
#http://www.ncbi.nlm.nih.gov/gorf/gorf.html. The HTTP::Request::Common
#module's POST method takes $url and the list of attributes and
#values, and makes a request object that is sent using $browser's
#request method.
my $response = $browser->request(
    POST "http://www.ncbi.nlm.nih.gov/gorf/orfig.cgi",
    [ SEQUENCE => $sequence, gcode => '1' ]
);

#$response is an object, which means it is a collection of values and
#methods. The different values it contains are described in the Perl
#documentation:
#http://search.cpan.org/author/GAAS/libwww-perl/lib/HTTP/Response.pm.
#We will use the "is_success()" method, which returns true if a
#response was received from the ORF Finder server.
my $result = undef;
if ( $response->is_success() ) {
    print "A response was received from the ORF Finder server.\n";

    #The "as_string()" method returns the contents of the response (the
    #web page) as a string.
    $result = $response->as_string();
}
else {
    die("A response was not received from the ORF Finder server.\n");
}

open( my $OUTFILE, '>', 'orf_finder_results.html' )
    or die("Cannot open file 'orf_finder_results.html': $!");
print $OUTFILE $result;
close($OUTFILE);

print
    "Your results have been written to a file called 'orf_finder_results.html'.\n";

#The following reads the contents of the html file and prints out
#information about each ORF. Note that the contents of $result could
#be parsed instead. The 'local $/ = "\n"' ensures that the "<>"
#operator, used in the line 'while (my $line = <$RESULTSFILE>) {',
#reads one line of the file at a time.
local $/ = "\n";
open( my $RESULTSFILE, 'orf_finder_results.html' )
    or die("Cannot open file 'orf_finder_results.html': $!");

#The while loop is used to read in each line of the ORF output file.
while (my $line = <$RESULTSFILE>) {

#Each ORF is described by html like the following (in the actual html
#file the following ORF description appears on a single line):
#<tr><td align=center>+1</td><td align=center><input type=image
#name=orf1 border=0 src=cyan.gif></td><td
#align=right>265</td><td>..</td></td><td
#align=right>13413</td></td><td align=right>13149</td></tr>

#We can use a regular expression to capture the reading frame, start
#position, end position, and length. In the above example these values
#are '+1', '265', '13413', and '13149', respectively. In the regular
#expression below, '([+-][123])' matches a plus or minus sign followed
#by a 1, 2, or 3. The parentheses cause the matching text to be stored
#in a variable called $1. The '(\d+)' portions each match one or more
#digits, and are used to match the start, end, and length values. In
#each case the matching text is stored in a variable ($2, $3, or
#$4). The other parts of the regular expression are used to match the
#flanking or interspersed html code. For example, the first '<.+>'
#matches a '<' character, followed by one more characters of any type,
#followed by a '>' character.

    if ($line =~ m/>([+-][123])<.+>(\d+)<.+>(\d+)<.+>(\d+)</) {
	print "reading frame=$1; start=$2; end=$3; length=$4\n";
    }
}

close($RESULTSFILE) or die("Cannot close file : $!");

#To run this script, use the following command:
#perl 1_find_orfs.pl
