#!/usr/local/bin/perl
#2_genscan.pl
#Author: Paul Stothard
#
#This script reads a single DNA sequence (in FASTA format) from a file
#and then sends the sequence to a Genscan web server. The predicted
#gene translations returned by Genscan are written to a file.

use warnings;
use strict;

#Perl comes with a group of modules that can be used to automate
#Web-related activities. We need to load two modules that we will be
#using.
use LWP::UserAgent;
use HTTP::Request::Common;

#Obtain the DNA sequence from a file.
print "Enter the name of the sequence file"
    . " (try genomic.txt) and then press enter.\n";
my $file_to_read = <STDIN>;
chomp($file_to_read);
open( my $DNAFILE, $file_to_read )
    or die("Cannot open file '$file_to_read': $!");
local $/ = undef;
my $direct_strand = <$DNAFILE>;
my $sequence_title;

if ( $direct_strand =~ m/>([^\n]+)/ ) {
    $sequence_title = $1;
}
else {
    die("A FASTA sequence title was not found.");
}

#Once you are done with a file, close the filehandle.
close($DNAFILE) or die("Cannot close file: $!");

#Store the Genscan URL we will be using in $url.
my $url = "http://genes.mit.edu/cgi-bin/oldgenscanw.cgi";

#First declare some variables for using Genscan.
my $organism               = "Arabidopsis";
my $suboptimal_exon_cutoff = "1.00";
my $print_options          = "Predicted CDS and peptides";

#An email address is not required, so we will set it to an empty
#string.
my $email = "";

#Create a user agent object of class LWP::UserAgent. The user agent
#object acts like a web browser, and we can use it to send requests
#for resources on the web. The user agent object returns the results
#of these requests to us in the form of a response object. In the
#following statement we are creating the user agent object using the
#"LWP::UserAgent->new()". We refer to this object using $browser. Thus
#we have created a virtual web browser that we can access using a name
#we gave it--$browser.
my $browser = LWP::UserAgent->new();

#Our virtual web browser has many attributes we can adjust. A list of
#these attributes is available at
#http://search.cpan.org/author/GAAS/libwww-perl/lib/LWP/UserAgent.pm
#We will change the timeout value of our browser, which is how long
#the browser will wait for a response before timing out (i.e.,
#canceling the request). We will set it to 30 seconds.
$browser->timeout(30);

#Send a request to the Genscan server. A request contains the
#information that a normal web browser sends to a server when it makes
#a request for a particular page. This information includes the URL of
#the page. In our case we also need to submit the information that the
#Genscan server expects. The following statement causes our virtual
#browser to send a request. The results of the request, the response,
#is stored in $response. The part of the request that the Genscan
#server sees is the list of attributes and values, starting with -o =
#> $organism, and ending with -a => $email. These attributes were
#identified by reading the page source of the Genscan web site at
#http://genes.mit.edu/oldGENSCAN.html The HTTP::Request::Common
#module's POST method takes $url and the list of attributes and
#values, and makes a request object that is sent using $browser's
#request method.
print "Sending a request to the Genscan server.\n";
my $response = $browser->request(
    POST $url,
    [   -o => $organism,
        -e => $suboptimal_exon_cutoff,
        -n => $sequence_title,
        -p => $print_options,
        -s => $direct_strand,
        -a => $email
    ]
);

#$response is an object, which means it is a collection of values and
#methods. The different values it contains are described in the Perl
#documentation:
#http://search.cpan.org/author/GAAS/libwww-perl/lib/HTTP/Response.pm.
#We will use the "is_success()" method, which returns true if a
#response was received from the Genscan server.
my $result = undef;
if ( $response->is_success() ) {
    print "A response was received from the Genscan server.\n";

    #The "as_string()" method returns the contents of the response (the
    #web page) as a string.
    $result = $response->as_string();
}
else {
    die("A response was not received from the Genscan server.\n");
}

#Now parse the results and store each entry in
#@array_of_translations. If you examine the output of Genscan in a web
#browser you will see that each predicted translation begins with a
#FASTA title like the following:
#>gi|GENSCAN_predicted_peptide_1|519_aa
#We can use the matching operator to separate the predicted
#translations from the rest of the html output.
my @array_of_translations = ();
while ( $result =~ m/(>gi\|GENSCAN_predicted_peptide_[^>]+)/g ) {
    my $entry = $1;

    #Get rid of extra blank spaces.
    $entry =~ s/\s\s+//g;
    $entry = $entry . "\n";
    push( @array_of_translations, $entry );
}

#Write the results to a file.
print "Writing results to genscan_results.txt\n";
open( my $OUTFILE, ">", "genscan_results.txt" )
    or die("Cannot open file 'genscan_results.txt': $!");
print $OUTFILE "Genscan results for \"$sequence_title\".\n";
print $OUTFILE "Length = " . length($direct_strand) . " bp.\n";
if ( scalar( @array_of_translations != 0 ) ) {
    print $OUTFILE "Genscan returned "
        . scalar(@array_of_translations)
        . " translations:\n";
}
else {
    print $OUTFILE "Genscan returned no translations.\n";
}
foreach (@array_of_translations) {
    print $OUTFILE "$_\n";
}

#Close the filehandle.
close($OUTFILE) or die("Cannot close file: $!");

#To run this script, use the following command:
#perl 2_genscan.pl
