#!/usr/local/bin/perl
#4_mw.pl
#Author: Paul Stothard
#
#This script reads multiple protein sequences (in FASTA format) from a
#file and determines the amino acid usage and molecular weight of each
#sequence, and the combined amino acid usage for all the
#sequences. The results are written to a file.

use warnings;
use strict;

#Create a hash table to use for the molecular weight
#calculation. Protein Mw is calculated by the addition of average
#isotopic masses of amino acids in the protein and the average
#isotopic mass of one water molecule.
my $water_mass            = 18.01524;
my %molecular_weight_hash = (
    A => 71.0788,
    C => 103.1388,
    D => 115.0886,
    E => 129.1155,
    F => 147.1766,
    G => 57.0519,
    H => 137.1411,
    I => 113.1594,
    K => 128.1741,
    L => 113.1594,
    M => 131.1926,
    N => 114.1038,
    P => 97.1167,
    Q => 128.1307,
    R => 156.1875,
    S => 87.0782,
    T => 101.1051,
    V => 99.1326,
    W => 186.2132,
    Y => 163.1760
);

#Create a hash table to hold the amino acid usage for a single
#sequence.
my %sequence_usage_hash = (
    A => 0,
    C => 0,
    D => 0,
    E => 0,
    F => 0,
    G => 0,
    H => 0,
    I => 0,
    K => 0,
    L => 0,
    M => 0,
    N => 0,
    P => 0,
    Q => 0,
    R => 0,
    S => 0,
    T => 0,
    V => 0,
    W => 0,
    Y => 0
);

#Create a hash table to hold the amino acid usage for all the
#sequences.
my %total_usage_hash = (
    A => 0,
    C => 0,
    D => 0,
    E => 0,
    F => 0,
    G => 0,
    H => 0,
    I => 0,
    K => 0,
    L => 0,
    M => 0,
    N => 0,
    P => 0,
    Q => 0,
    R => 0,
    S => 0,
    T => 0,
    V => 0,
    W => 0,
    Y => 0
);

#Create a file for recording results.
open( my $OUTFILE, ">", "mw_results.txt" )
    or die("Cannot open file 'mw_results.txt': $!");
print $OUTFILE "Molecular weight and amino acid usage\n";
print $OUTFILE "-------------------------------------\n";

#Prompt the user for the name of the file to read.
print "Enter the name of a protein sequence file"
    . " (try proteins.txt) and then press enter:\n";
my $file_to_read = <STDIN>;
chomp($file_to_read);
open( my $PROTEINFILE, $file_to_read )
    or die("Cannot open file '$file_to_read': $!");
local $/ = ">";

#Read each FASTA entry and determine the molecular weight and amino
#acid usage. Rather than first storing all the sequence names and
#sequences in arrays, we will process each sequence completely before
#moving on to the next one. This method is better for large files
#because only the sequence being processed needs to be stored in
#memory.
while ( my $sequence_entry = <$PROTEINFILE> ) {

    #Declare a variable to store the molecular weight of the sequence.
    my $molecular_weight = 0;

    if ( $sequence_entry eq ">" ) {
        next;
    }
    my $sequence_title = "";
    if ( $sequence_entry =~ m/^([^\n]+)/ ) {
        $sequence_title = $1;
    }
    else {
        $sequence_title = "No title was found!";
    }

    #This removes the title.
    $sequence_entry =~ s/^[^\n]+//;

    #Remove any characters that are not between a-z or A-Z.
    $sequence_entry =~ s/[^a-z]//ig;

    #Check to see if there is are any characters in the protein
    #sequence. If there aren't, skip the sequence.
    if ( length($sequence_entry) == 0 ) {
        next;
    }

    #Check to see if the sequence contains any unrecognized amino
    #acids. If it does we will stop the program.
    if ( $sequence_entry =~ m/[^ACDEFGHIKLMNPQRSTVWY]/i ) {
        die("An unknown amino acid was encountered in entry $sequence_title.\n"
        );
    }

    #Now convert the protein sequence to an array.
    my @array_of_amino_acids = split( //, $sequence_entry );

    #Go through all the amino acids. For each one obtain the molecular
    #weight from %molecular_weight_hash, and add an entry to the hash tables
    #%sequence_usage_hash and %total_usage_hash.
    foreach (@array_of_amino_acids) {

        #$_ will refer to the amino acid.
        $molecular_weight = $molecular_weight + $molecular_weight_hash{$_};
        $sequence_usage_hash{$_} = $sequence_usage_hash{$_} + 1;
        $total_usage_hash{$_}    = $total_usage_hash{$_} + 1;
    }

    #Write the results for the entry to the file.
    $molecular_weight = $molecular_weight + $water_mass;

    #sprintf returns the value stored in $molecular_weight after rounding
    #it to two decimal places.
    my $formatted_molecular_weight = sprintf( "%.2f", $molecular_weight );
    print $OUTFILE ">$sequence_title\n";
    print $OUTFILE "Molecular weight = $formatted_molecular_weight\n";
    print $OUTFILE "Amino acid usage: ";
    my @hash_keys = keys(%sequence_usage_hash);

    #The keys function does not necessarily return the keys in the order
    #they are defined in the hash. We will sort them so that our output is
    #in alphabetical order.
    @hash_keys = sort(@hash_keys);
    foreach (@hash_keys) {
        print $OUTFILE $_ . "=" . $sequence_usage_hash{$_} . " ";

        #Set each value to zero so that the next sequence can use the hash.
        $sequence_usage_hash{$_} = 0;
    }
    print $OUTFILE "\n\n";
}    #end of while loop for reading sequences.
close($PROTEINFILE) or die("Cannot close file: $!");

#Print the total amino acid usage to the file.
print $OUTFILE "----------------------------------------------\n";
print $OUTFILE "The amino acid usage for all the sequences is:\n";
my @hash_keys = keys(%total_usage_hash);
@hash_keys = sort(@hash_keys);
foreach (@hash_keys) {
    print $OUTFILE $_ . "=" . $total_usage_hash{$_} . " ";
}
print $OUTFILE "\n";
close($OUTFILE) or die("Cannot close file: $!");
print "Open mw_results.txt to view the results.\n";

#To run this script, use the following command:
#perl 4_mw.pl
